@@ -184,7 +184,7 @@ import org.apache.commons.pool.impl.GenericKeyedObjectPool.ObjectTimestampPair;
  * @author Rodney Waldhoff
  * @author Dirk Verbeeck
  * @author Sandy McArthur
- * @version $Revision: 778428 $ $Date: 2009-05-25 10:34:21 -0400 (Mon, 25 May 2009) $
+ * @version $Revision: 784441 $ $Date: 2009-06-13 13:42:28 -0400 (Sat, 13 Jun 2009) $
  * @since Pool 1.0
  */
 public class GenericObjectPool extends BaseObjectPool implements ObjectPool {
@@ -1074,12 +1074,21 @@ public class GenericObjectPool extends BaseObjectPool implements ObjectPool {
                         case WHEN_EXHAUSTED_GROW:
                             // allow new object to be created
                             synchronized (this) {
-                                _allocationQueue.remove(latch);
-                                _numInternalProcessing++;
+                                // Make sure allocate hasn't already assigned an object
+                                // in a different thread
+                                if (latch.getPair() == null) {
+                                    _allocationQueue.remove(latch);
+                                    _numInternalProcessing++;
+                                }
                             }
                             break;
                         case WHEN_EXHAUSTED_FAIL:
                             synchronized (this) {
+                                // Make sure allocate hasn't already assigned an object
+                                // in a different thread
+                                if (latch.getPair() != null) {
+                                    break;
+                                }
                                 _allocationQueue.remove(latch);
                             }
                             throw new NoSuchElementException("Pool exhausted");
@@ -1104,6 +1113,16 @@ public class GenericObjectPool extends BaseObjectPool implements ObjectPool {
                                 throw e;
                             }
                             if(maxWait > 0 && ((System.currentTimeMillis() - starttime) >= maxWait)) {
+                                synchronized(this) {
+                                    // Make sure allocate hasn't already assigned an object
+                                    // in a different thread
+                                    if (latch.getPair() == null) {
+                                        // Remove latch from the allocation queue
+                                        _allocationQueue.remove(latch);
+                                    } else {
+                                        break;
+                                    }
+                                }
                                 throw new NoSuchElementException("Timeout waiting for idle object");
                             } else {
                                 continue; // keep looping
